from datetime import datetime

from cpl_core.configuration import ConfigurationModelABC
from cpl_core.database import TableABC
from cpl_query.extension import List

from bot_data.model.server import Server
from bot_data.model.server_team_role_ids_config import ServerTeamRoleIdsConfig


class ServerConfig(TableABC, ConfigurationModelABC):
    def __init__(
        self,
        message_delete_timer: int,
        notification_chat_id: int,
        max_voice_state_hours: int,
        xp_per_message: int,
        xp_per_reaction: int,
        max_message_xp_per_hour: int,
        xp_per_ontime_hour: int,
        xp_per_event_participation: int,
        xp_per_achievement: int,
        afk_command_channel_id: int,
        help_voice_channel_id: int,
        team_channel_id: int,
        login_message_channel_id: int,
        server: Server,
        afk_channel_ids: List[int],
        team_role_ids: List[ServerTeamRoleIdsConfig],
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._id = id
        self._message_delete_timer = message_delete_timer
        self._notification_chat_id = notification_chat_id
        self._max_voice_state_hours = max_voice_state_hours
        self._xp_per_message = xp_per_message
        self._xp_per_reaction = xp_per_reaction
        self._max_message_xp_per_hour = max_message_xp_per_hour
        self._xp_per_ontime_hour = xp_per_ontime_hour
        self._xp_per_event_participation = xp_per_event_participation
        self._xp_per_achievement = xp_per_achievement
        self._afk_command_channel_id = afk_command_channel_id
        self._help_voice_channel_id = help_voice_channel_id
        self._team_channel_id = team_channel_id
        self._login_message_channel_id = login_message_channel_id
        self._server = server
        self._afk_channel_ids = afk_channel_ids
        self._team_role_ids = team_role_ids

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._id

    @property
    def message_delete_timer(self) -> int:
        return self._message_delete_timer

    @message_delete_timer.setter
    def message_delete_timer(self, value: int):
        self._message_delete_timer = value

    @property
    def notification_chat_id(self) -> int:
        return self._notification_chat_id

    @notification_chat_id.setter
    def notification_chat_id(self, value: int):
        self._notification_chat_id = value

    @property
    def max_voice_state_hours(self) -> int:
        return self._max_voice_state_hours

    @max_voice_state_hours.setter
    def max_voice_state_hours(self, value: int):
        self._max_voice_state_hours = value

    @property
    def xp_per_message(self) -> int:
        return self._xp_per_message

    @xp_per_message.setter
    def xp_per_message(self, value: int):
        self._xp_per_message = value

    @property
    def xp_per_reaction(self) -> int:
        return self._xp_per_reaction

    @xp_per_reaction.setter
    def xp_per_reaction(self, value: int):
        self._xp_per_reaction = value

    @property
    def max_message_xp_per_hour(self) -> int:
        return self._max_message_xp_per_hour

    @max_message_xp_per_hour.setter
    def max_message_xp_per_hour(self, value: int):
        self._max_message_xp_per_hour = value

    @property
    def xp_per_ontime_hour(self) -> int:
        return self._xp_per_ontime_hour

    @xp_per_ontime_hour.setter
    def xp_per_ontime_hour(self, value: int):
        self._xp_per_ontime_hour = value

    @property
    def xp_per_event_participation(self) -> int:
        return self._xp_per_event_participation

    @xp_per_event_participation.setter
    def xp_per_event_participation(self, value: int):
        self._xp_per_event_participation = value

    @property
    def xp_per_achievement(self) -> int:
        return self._xp_per_achievement

    @xp_per_achievement.setter
    def xp_per_achievement(self, value: int):
        self._xp_per_achievement = value

    @property
    def afk_command_channel_id(self) -> int:
        return self._afk_command_channel_id

    @afk_command_channel_id.setter
    def afk_command_channel_id(self, value: int):
        self._afk_command_channel_id = value

    @property
    def help_voice_channel_id(self) -> int:
        return self._help_voice_channel_id

    @help_voice_channel_id.setter
    def help_voice_channel_id(self, value: int):
        self._help_voice_channel_id = value

    @property
    def team_channel_id(self) -> int:
        return self._team_channel_id

    @team_channel_id.setter
    def team_channel_id(self, value: int):
        self._team_channel_id = value

    @property
    def login_message_channel_id(self) -> int:
        return self._login_message_channel_id

    @login_message_channel_id.setter
    def login_message_channel_id(self, value: int):
        self._login_message_channel_id = value

    @property
    def afk_channel_ids(self) -> List[int]:
        return self._afk_channel_ids

    @afk_channel_ids.setter
    def afk_channel_ids(self, value: List[int]):
        self._afk_channel_ids = value

    @property
    def team_role_ids(self) -> List[ServerTeamRoleIdsConfig]:
        return self._team_role_ids

    @team_role_ids.setter
    def team_role_ids(self, value: List[ServerTeamRoleIdsConfig]):
        self._team_role_ids = value

    @property
    def server(self) -> Server:
        return self._server

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
                SELECT * FROM `CFG_Server`;
            """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
                SELECT * FROM `CFG_Server`
                WHERE `Id` = {id};
            """
        )

    @staticmethod
    def get_select_by_server_id_string(server_id: int) -> str:
        return str(
            f"""
                SELECT * FROM `CFG_Server`
                WHERE `ServerId` = {server_id};
            """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
                INSERT INTO `CFG_Server` (
                    `MessageDeleteTimer`,
                    `NotificationChatId`,
                    `MaxVoiceStateHours`,
                    `XpPerMessage`,
                    `XpPerReaction`,
                    `MaxMessageXpPerHour`,
                    `XpPerOntimeHour`,
                    `XpPerEventParticipation`,
                    `XpPerAchievement`,
                    `AFKCommandChannelId`,
                    `HelpVoiceChannelId`,
                    `TeamChannelId`,
                    `LoginMessageChannelId`,
                    `ServerId`
                ) VALUES (
                    {self._message_delete_timer},
                    {self._notification_chat_id},
                    {self._max_voice_state_hours},
                    {self._xp_per_message},
                    {self._xp_per_reaction},
                    {self._max_message_xp_per_hour},
                    {self._xp_per_ontime_hour},
                    {self._xp_per_event_participation},
                    {self._xp_per_achievement},
                    {self._afk_command_channel_id},
                    {self._help_voice_channel_id},
                    {self._team_channel_id},
                    {self._login_message_channel_id},
                    {self._server.id}
                );
            """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
                UPDATE `CFG_Server`
                SET `MessageDeleteTimer` = {self._message_delete_timer},
                `NotificationChatId` = {self._notification_chat_id},
                `MaxVoiceStateHours` = {self._max_voice_state_hours},
                `XpPerMessage` = {self._xp_per_message},
                `XpPerReaction` = {self._xp_per_reaction},
                `MaxMessageXpPerHour` = {self._max_message_xp_per_hour},
                `XpPerOntimeHour` = {self._xp_per_ontime_hour},
                `XpPerEventParticipation` = {self._xp_per_event_participation},
                `XpPerAchievement` = {self._xp_per_achievement},
                `AFKCommandChannelId` = {self._afk_command_channel_id},
                `HelpVoiceChannelId` = {self._help_voice_channel_id},
                `TeamChannelId` = {self._team_channel_id},
                `LoginMessageChannelId` = {self._login_message_channel_id},
                `ServerId` = {self._server.id}
                WHERE `Id` = {self._id};
            """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
                DELETE FROM `CFG_Server`
                WHERE `Id` = {self._id};
            """
        )
