from cpl_core.database.context import DatabaseContextABC
from cpl_discord.container import Guild
from cpl_discord.service import DiscordBotServiceABC

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.data_seeder_abc import DataSeederABC
from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_config import ServerConfig


class ServerConfigSeeder(DataSeederABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        server_config: ServerConfigRepositoryABC,
        db: DatabaseContextABC,
    ):
        DataSeederABC.__init__(self)

        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._server_config = server_config
        self._db = db

    async def seed(self):
        try:
            for guild in self._bot.guilds:
                guild: Guild = guild
                server = self._servers.get_server_by_discord_id(guild.id)
                if self._server_config.does_server_config_exists(server.id):
                    continue

                config = ServerConfig(
                    6,
                    guild.system_channel.id,
                    6,
                    1,
                    1,
                    20,
                    10,
                    10,
                    10,
                    guild.system_channel.id,
                    guild.system_channel.id,
                    guild.system_channel.id,
                    guild.system_channel.id,
                    server,
                    [],
                    [],
                )

                self._server_config.add_server_config(config)

                self._db.save_changes()
                self._logger.debug(__name__, "Seeded technician config")

        except Exception as e:
            self._logger.error(__name__, f"Seeding technician config failed", e)
