from cpl_discord.service import DiscordBotServiceABC

from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import UserJoinedGameServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import UserJoinedVoiceChannelRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_graphql.abc.query_abc import QueryABC
from bot_graphql.filter.achievement_filter import AchievementFilter
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.auto_role_rule_filter import AutoRoleRuleFilter
from bot_graphql.filter.client_filter import ClientFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.server_filter import ServerFilter
from bot_graphql.filter.user_filter import UserFilter
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import UserJoinedVoiceChannelFilter
from modules.achievements.achievement_service import AchievementService


class Query(QueryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        known_users: KnownUserRepositoryABC,
        levels: LevelRepositoryABC,
        servers: ServerRepositoryABC,
        game_servers: GameServerRepositoryABC,
        user_joined_servers: UserJoinedServerRepositoryABC,
        user_joined_voice_channels: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        users: UserRepositoryABC,
        achievements: AchievementRepositoryABC,
        achievement_service: AchievementService,
        technician_config: TechnicianConfigRepositoryABC,
    ):
        QueryABC.__init__(self, "Query")

        self._bot = bot

        self.add_collection("autoRole", lambda *_: auto_roles.get_auto_roles(), AutoRoleFilter)
        self.add_collection("autoRoleRule", lambda *_: auto_roles.get_auto_role_rules(), AutoRoleRuleFilter)
        self.add_collection("client", lambda *_: clients.get_clients(), ClientFilter)
        self.add_collection("knownUser", lambda *_: known_users.get_users())
        self.add_collection("level", lambda *_: levels.get_levels(), LevelFilter)
        self.add_collection("server", lambda *_: servers.get_servers(), ServerFilter)
        self.add_collection("gameServer", lambda *_: game_servers.get_game_servers())
        self.add_collection(
            "userJoinedServer", lambda *_: user_joined_servers.get_user_joined_servers(), UserJoinedServerFilter
        )
        self.add_collection(
            "userJoinedVoiceChannel",
            lambda *_: user_joined_voice_channels.get_user_joined_voice_channels(),
            UserJoinedVoiceChannelFilter,
        )
        self.add_collection(
            "userJoinedGameServer",
            lambda *_: user_joined_game_server.get_user_joined_game_servers(),
            UserJoinedGameServerFilter,
        )
        self.add_collection("user", lambda *_: users.get_users(), UserFilter)
        self.add_collection("achievement", lambda *_: achievements.get_achievements(), AchievementFilter)
        self.set_field("technicianConfig", lambda *_: technician_config.get_technician_config())

        self.set_field("guilds", self._resolve_guilds)
        self.set_field("achievementAttributes", lambda x, *_: achievement_service.get_attributes())
        self.set_field("achievementOperators", lambda x, *_: achievement_service.get_operators())

    def _resolve_guilds(self, *_, filter=None):
        if filter is None or "id" not in filter:
            return self._bot.guilds

        return self._bot.guilds.where(lambda g: g.id == int(filter["id"]))
