import discord
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.achievements.achievement_service import AchievementService


class AchievementGroup(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        achievement_service: AchievementService,
        translate: TranslatePipe,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._servers = servers
        self._users = users
        self._achievement_service = achievement_service
        self._t = translate

    @commands.hybrid_group()
    @commands.guild_only()
    async def achievement(self, ctx: Context):
        pass

    @achievement.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def check(self, ctx: Context, member: discord.Member):
        self._logger.debug(__name__, f"Received command achievement check {ctx}")

        server = self._servers.get_server_by_discord_id(member.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        await self._message_service.send_ctx_msg(
            ctx,
            self._t.transform("modules.achievements.commands.check"),
        )
        await self._achievement_service.validate_achievements_for_user(user)

        self._logger.trace(__name__, f"Finished command achievement check")
