import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_translation import TranslatePipe
from discord import ui, TextStyle

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from bot_data.model.server_config import ServerConfig


class ComplaintForm(ui.Modal):
    description = ui.TextInput(label="Complain about something", required=True, style=TextStyle.long)

    def __init__(
        self,
        config: ConfigurationABC,
        db: DatabaseContextABC,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        t: TranslatePipe,
    ):
        ui.Modal.__init__(self, title=t.transform("modules.base.complaints.title"))

        self._config = config
        self._db = db
        self._message_service = message_service
        self._logger = logger
        self._t = t

        self.description.label = t.transform("modules.base.complaints.label")

    async def on_submit(self, interaction: discord.Interaction):
        self._logger.debug(__name__, f"Started complaint command form")
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{interaction.guild.id}")
        channel = interaction.guild.get_channel(settings.team_channel_id)
        await self._message_service.send_channel_message(
            channel,
            self._t.transform("modules.base.complaints.message").format(
                interaction.user.mention, self.description.value
            ),
            is_persistent=True,
        )
        await self._message_service.send_interaction_msg(
            interaction, self._t.transform("modules.base.complaints.response")
        )
        self._logger.trace(__name__, f"Finished complaint command form")
