from cpl_core.application.application_extension_abc import ApplicationExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.logging import LoggerABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.technician_config import TechnicianConfig


class ConfigExtension(ApplicationExtensionABC):
    def __init__(self):
        pass

    async def run(self, config: ConfigurationABC, services: ServiceProviderABC):
        feature_flags: FeatureFlagsSettings = config.get_configuration(FeatureFlagsSettings)
        if not feature_flags.get_flag(FeatureFlagsEnum.config_module):
            return
        logger: LoggerABC = services.get_service(LoggerABC)
        logger.debug(__name__, "Config extension started")
        technician_config_repo: TechnicianConfigRepositoryABC = services.get_service(TechnicianConfigRepositoryABC)
        technician_config = technician_config_repo.get_technician_config()
        config.add_configuration(TechnicianConfig, technician_config)
